# coding: utf-8
require 'rack/response'
require 'uri'
require 'time'
require 'base64'

class VCClientEntry

  SECURE = false

  def get_dest_url(params)
    if params.nil? then
        return '/'
    end

    view_url = params['VIEW_URL'].nil? ? '/' : params['VIEW_URL']
    vc_url = params['VC_URL'].nil? ? '' : params['VC_URL']

    if vc_url.empty? then
      return view_url
    end
    return vc_url
  end

  def gen_redirect_html(vclpp, dest_url)
    html = '<html><head></head><body></body>' \
           '<script>var vc_lpp = "' + URI.encode_www_form_component(vclpp).gsub(/\+/, '%20') + '"; var dest_url = "' + dest_url + '";</script>' \
           '<script type="text/javascript" src="//cej.valuecommerce.com/vcce.js"></script>' \
           '<script>if("function"!=typeof vcIBeacon){window.location.replace("' + dest_url + '")}</script>' \
           '</html>'
  end

  def set_vccookies(response, params, domain, is_secure)
    if params.nil? || params['VC_LPP'].nil? || params['VC_LPP'].empty? then
      return
    end

    begin
      vclpp = Base64.urlsafe_decode64(params['VC_LPP'])
    rescue
      return
    end

    now = Time.now.to_i
    # split with ''
    vclpp.split(['02'].pack('H*')).each do |param|
      set_vccookie(response, param, domain, now, is_secure)
    end
    return vclpp
  end

  def set_vccookie(response, param, domain, now, is_secure)
    param_arr = param.split('&')
    if param_arr.length < 5 then
      return
    end

    hexreg = /\A[0-9A-Fa-f]+\z/ # is consisted of hex characters
    if !(hexreg === param_arr[2] && hexreg === param_arr[3] && hexreg === param_arr[4]) then
      return
    end

    sig_exp = param_arr[2].hex + 10800
    if now > sig_exp then
      return
    end

    ckey = '_vc_cec_' + param_arr[3].hex.to_s
    cval = Hash.new
    cval[:value]   = '2&' + param_arr[5]
    cval[:path]    = '/'
    cval[:expires] = Time.at(param_arr[4].hex)
    cval[:secure]  = is_secure
    cval[:httponly]= true
    darr = domain.split('.')
    dlen = darr.length
    if dlen > 3 then dlen = 3 end
    dlen.downto(2) do |len|
      cval[:domain] = '.' + darr.slice(-len..-1).join('.')
      response.set_cookie(ckey, cval)
    end
  end

  def get_upcase_keys_hash_from_qstr(qstr)
    q_hash = Hash.new
    URI.decode_www_form(qstr).each do |k, v|
      q_hash[k.upcase] = v
    end
    return q_hash
  end

  def call(env)
    params = get_upcase_keys_hash_from_qstr(env['QUERY_STRING'])
    domain = env['HTTP_HOST'].split(':')[0]

    req = Rack::Request.new(env)
    is_secure = SECURE ? SECURE : req.ssl?

    res = Rack::Response.new
    vclpp = set_vccookies(res, params, domain, is_secure)
    begin
      res.set_header('Content-Type', 'text/html; charset=utf-8')
    rescue # for old rack version
      res.[]=('Content-Type', 'text/html; charset=utf-8')
    end
    res.write(gen_redirect_html(vclpp, get_dest_url(params)))
    res.finish
  end
end
